﻿using System;

namespace CollectionInterfaceDelegateTest
{
    public class Person : IComparable<Person>, IEquatable<Person>
    {
        protected string name;
        protected int id;
        protected string phone;
        protected string address;

        public string Name
        {
            get
            {
                return name;
            }
            set
            {
                name = value;
            }
        }

        public int ID
        {
            get
            {
                return id;
            }
            set
            {
                id = value;
            }
        }

        public string Phone
        {
            get
            {
                return phone;
            }
            set
            {
                phone = value;
            }
        }

        public string Address
        {
            get
            {
                return address;
            }
            set
            {
                address = value;
            }
        }

        public Person() : this("", 0, "", "")
        {
        }

        public Person(string name, int id, string phone, string address)
        {
            this.name = name;
            this.id = id;
            this.phone = phone;
            this.address = address;
        }

        public Person(Person p)
        {
            this.name = p.name;
            this.id = p.id;
            this.phone = p.phone;
            this.address = p.address;
        }

        public virtual void Print()
        {
            Console.WriteLine("이름: {0} ID: {1} 전화번호: {2} 주소: {3}", this.name, this.id, this.phone, this.address);
        }

        // System.Object.ToString method override
        public override string ToString()
        {
            return string.Format("{0},{1},{2},{3}", Name, ID, Phone, Address); // csv file format으로 출력
        }

        // needed for sort
        #region IComparable<Person> PersonSort
        public int CompareTo(Person other)
        {
            return Name.CompareTo(other.Name);
        }
        #endregion

        // ==
        public static bool operator ==(Person p, Person q)
        {
            return p.Equals(q);
        }

        public static bool operator !=(Person p, Person q)
        {
            return !p.Equals(q);
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }
        
        public override bool Equals(object obj)
        {
            return Equals((Person)obj);
        }

        public bool Equals(Person other)
        {
            if (object.ReferenceEquals(this, other)) return true;   // 효율을 위해, 두 참조자가 같으면 true
            return (Name == other.Name) && (ID == other.ID) && (Phone == other.Phone) && (Address == other.Address);
        }
    }
}
